#include <math.h>
#include <malloc.h>

#define MSIZE 2

void main()
{
/*

  LU Decomposition:
  Reference: Numerical Recipes in C, pp. 39-45

  This program solves equations of the form Ax=b

  LUDCMP:
  void ludcmp(a, n, indx, d)
  Given an n x n matrix a[1..n][1..n], this routine replaces it by
  the LU decomposition of a row-wise permutation of itself. a and
  n are input. a is output in which the rows are arranged in a 
  particular order. indx[1..n] is an output vector which records the
  row permutation effected by partial pivoting. d is an output as
  +1 or -1, depending on whether the number of row interchanges was
  even or odd respectively. This routine is used in combination with
  lubksb to solve linear equations or invert a matrix.

  LUBKSB
  void lubksb(a, n, indx, b)
  Solves a set of linear equations A.X = B. Here a[1..n][1..n] is 
  input, not as the input matrix A but as its LU decomposition, 
  determined by the routine ludcmp. b[1..n] is input as the 
  right-hand side vector B,, and returns with the solution vector
  X. a, n, and indx are not modified by this routine  and can be
  left in place for successive calls with different right hand sides 
  b. This routine takes into account the possiblitiy that b will begin 
  with many zero elements so it is efficient for use with matrix 
  inverson too.


  Solving a set of linear equations A x = b
  
  float **a, *b, d;
  int n, *indx;
  ....		mem allocation and read a, b, code here
  ludcmp(a,n,indx,&d);
  lubksb(a,n,indx,b);

  The anser x will be given back in b. The original matrix A would be 
  destroyed. If another system is to be solved with the same A but
  different b, repeat ONLY lubksb(a,n,indx,b); Use the A that was 
  returned by the first call to lubksb

  The small example program below solves the linear system A x = b for
  A =  [1 2
		3 4]
  b =  [2
		5]

  IMPORTANT NOTE:
  the routines are written such that the data should be in index 1..n and
  NOT 0..n-1. i.e. the top right element of matrix A is at location a[1][1].
  The bottom left element is at location a[n][n], where n is the size of
  the matrix. Therefore, arguments to malloc and calloc are n+1;
*/



	float **a, *b, d;
	int n, *indx;
	int i;

	/* Memory Allocation */
	a=(float**)malloc(sizeof(float*)*(MSIZE+1));
	for (i=0;i<=MSIZE;i++)
	{
		a[i]=(float*)calloc(sizeof(float),(MSIZE+1));
	}
	indx=(int*)calloc(sizeof(int),(MSIZE+1));
	b=(float*)calloc(sizeof(float),(MSIZE+1));


	/* Test Data */
	a[1][1]=1.0;
	a[1][2]=2.0;
	a[2][1]=3.0;
	a[2][2]=4.0;

	n=MSIZE;

	b[1]=2;
	b[2]=5;

	/* LU Decomposition Routines */
	ludcmp(a,n,indx,&d);
	lubksb(a,n,indx,b);


	/* deallocate mem */
	free(b);
	free(indx);
	for (i = 0;i<=MSIZE; i++)
		free(a[i]);
	free(a);
}